package com.seikoinstruments.sdk.mobileprinter.sample;

import android.app.Dialog;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.Spinner;

import com.seikoinstruments.sdk.mobileprinter.Alignment;
import com.seikoinstruments.sdk.mobileprinter.HriPositionBarcode;
import com.seikoinstruments.sdk.mobileprinter.ModuleWidthBarcode;
import com.seikoinstruments.sdk.mobileprinter.NwRatioBarcode;
import com.seikoinstruments.sdk.mobileprinter.PrinterException;
import com.seikoinstruments.sdk.mobileprinter.TypeBarcode;

public class BarcodeDialogFragment extends CommandDialogFragment
        implements CommandDialogFragment.RequestSender {

    private EditText mEditTextData;
    private EditText mEditTextModuleHeight;

    private TypeBarcode mType;
    private String mData;
    private ModuleWidthBarcode mModuleWidth;
    private int mModuleHeight;
    private HriPositionBarcode mHriPosition;
    private NwRatioBarcode mNwRatio = NwRatioBarcode.NWRATIO_BARCODE_1TO2;

    private static final TypeBarcode[] sType = new TypeBarcode[] {
            TypeBarcode.TYPE_BARCODE_UPC_A,
            TypeBarcode.TYPE_BARCODE_UPC_E,
            TypeBarcode.TYPE_BARCODE_JAN13,
            TypeBarcode.TYPE_BARCODE_JAN8,
            TypeBarcode.TYPE_BARCODE_CODE39,
            TypeBarcode.TYPE_BARCODE_ITF,
            TypeBarcode.TYPE_BARCODE_CODABAR,
            TypeBarcode.TYPE_BARCODE_CODE128,
            TypeBarcode.TYPE_BARCODE_CODE93,
            TypeBarcode.TYPE_BARCODE_JAN13_ADDON2,
            TypeBarcode.TYPE_BARCODE_JAN13_ADDON5,
            TypeBarcode.TYPE_BARCODE_GS1_OMNI_DIRECTIONAL,
            TypeBarcode.TYPE_BARCODE_GS1_TRUNCATED,
            TypeBarcode.TYPE_BARCODE_GS1_LIMITED,
            TypeBarcode.TYPE_BARCODE_GS1_EXPANDED
    };

    private static final ModuleWidthBarcode[] sModuleWidth = new ModuleWidthBarcode[] {
            ModuleWidthBarcode.MODULE_WIDTH_BARCODE_2,
            ModuleWidthBarcode.MODULE_WIDTH_BARCODE_3,
            ModuleWidthBarcode.MODULE_WIDTH_BARCODE_4,
            ModuleWidthBarcode.MODULE_WIDTH_BARCODE_5,
            ModuleWidthBarcode.MODULE_WIDTH_BARCODE_6
    };

    private static final HriPositionBarcode[] sHriPosition = new HriPositionBarcode[] {
            HriPositionBarcode.HRI_BARCODE_NONE,
            HriPositionBarcode.HRI_BARCODE_TOP_FONT_A,
            HriPositionBarcode.HRI_BARCODE_BOTTOM_FONT_A,
            HriPositionBarcode.HRI_BARCODE_FONT_A,
            HriPositionBarcode.HRI_BARCODE_TOP_FONT_B,
            HriPositionBarcode.HRI_BARCODE_BOTTOM_FONT_B,
            HriPositionBarcode.HRI_BARCODE_FONT_B
    };

    private AdapterView.OnItemSelectedListener mBarcodeTypeListener =
            new AdapterView.OnItemSelectedListener() {
        @Override
        public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
            mType = sType[position];
            replaceData(mType);
        }

        @Override
        public void onNothingSelected(AdapterView<?> parent) {
            mType = sType[0];
        }

        private void replaceData(TypeBarcode code) {
            switch (code) {
                case TYPE_BARCODE_UPC_A:
                    mEditTextData.setText("012345678905");
                    break;
                case TYPE_BARCODE_UPC_E:
                    mEditTextData.setText("01200000123");
                    break;
                case TYPE_BARCODE_JAN13:
                    mEditTextData.setText("012345678912");
                    break;
                case TYPE_BARCODE_JAN8:
                    mEditTextData.setText("0123456");
                    break;
                case TYPE_BARCODE_CODE39:
                    mEditTextData.setText("A");
                    break;
                case TYPE_BARCODE_ITF:
                    mEditTextData.setText("01");
                    break;
                case TYPE_BARCODE_CODABAR:
                    mEditTextData.setText("B+C");
                    break;
                case TYPE_BARCODE_CODE128:
                    mEditTextData.setText("{A0123456789");
                    break;
                case TYPE_BARCODE_CODE93:
                    mEditTextData.setText("0123456789");
                    break;
                case TYPE_BARCODE_JAN13_ADDON2:
                    mEditTextData.setText("49012345678940");
                    break;
                case TYPE_BARCODE_JAN13_ADDON5:
                    mEditTextData.setText("49012345678940123");
                    break;
                case TYPE_BARCODE_GS1_OMNI_DIRECTIONAL:
                    mEditTextData.setText("4901234567894");
                    break;
                case TYPE_BARCODE_GS1_TRUNCATED:
                    mEditTextData.setText("4901234567894");
                    break;
                case TYPE_BARCODE_GS1_LIMITED:
                    mEditTextData.setText("0123456789012");
                    break;
                case TYPE_BARCODE_GS1_EXPANDED:
                    mEditTextData.setText("490{123456789");
                    break;
                default:
                    mEditTextData.setText("012345678905");
                    break;
            }
        }
    };

    private AdapterView.OnItemSelectedListener mModuleWidthListener =
            new AdapterView.OnItemSelectedListener() {
        @Override
        public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
            mModuleWidth = sModuleWidth[position];
        }

        @Override
        public void onNothingSelected(AdapterView<?> parent) {
            mModuleWidth = ModuleWidthBarcode.MODULE_WIDTH_BARCODE_2;
        }
    };

    private AdapterView.OnItemSelectedListener mHriSettingListener =
            new AdapterView.OnItemSelectedListener() {
        @Override
        public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
            mHriPosition = sHriPosition[position];
        }

        @Override
        public void onNothingSelected(AdapterView<?> parent) {
            mHriPosition = sHriPosition[0];
        }
    };

    private View.OnClickListener mPrintListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            try {
                mData = getData();
                mModuleHeight = getModuleHeight();
                mNwRatio = getNwRatio();
                doRequest(false);
            } catch (NumberFormatException e) {
                popDialog(e.getClass().getSimpleName(), e.getMessage());
            }
        }
    };

    public static BarcodeDialogFragment newInstance(ConnectionInformation info) {
        BarcodeDialogFragment fragment = new BarcodeDialogFragment();
        Bundle args = new Bundle();
        args.putSerializable(MainActivity.CONN_INFO, info);
        fragment.setArguments(args);

        return fragment;
    }

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        mRequestSender = this;
        setLayout(getActivity().getLayoutInflater().inflate(R.layout.barcode_dialog, null));
        return super.onCreateDialog(savedInstanceState);
    }

    @Override
    protected void setupWidgets() {
        // Barcode Type
        Spinner spType = (Spinner) mLayout.findViewById(R.id.sp_barcode_type);
        ArrayAdapter<CharSequence> typeAdapter = ArrayAdapter.createFromResource(
                getActivity(), R.array.barcode_type, android.R.layout.simple_spinner_item);
        typeAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spType.setAdapter(typeAdapter);
        spType.setOnItemSelectedListener(mBarcodeTypeListener);

        // Data
        mEditTextData = (EditText) mLayout.findViewById(R.id.et_barcode_data);

        // Module Width
        Spinner spModuleWidth = (Spinner) mLayout.findViewById(R.id.sp_barcode_module_width);
        ArrayAdapter<CharSequence> widthAdapter = ArrayAdapter.createFromResource(
                getActivity(), R.array.module_width, android.R.layout.simple_spinner_item);
        widthAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spModuleWidth.setAdapter(widthAdapter);
        spModuleWidth.setOnItemSelectedListener(mModuleWidthListener);

        // Module Height
        mEditTextModuleHeight = (EditText) mLayout.findViewById(R.id.et_barcode_module_height);

        // HRI Setting
        Spinner spHriSetting = (Spinner) mLayout.findViewById(R.id.sp_barcode_hri_setting);
        ArrayAdapter<CharSequence> hriAdapter = ArrayAdapter.createFromResource(
                getActivity(), R.array.hri_setting, android.R.layout.simple_spinner_item);
        hriAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spHriSetting.setAdapter(hriAdapter);
        spHriSetting.setOnItemSelectedListener(mHriSettingListener);

        // print and close button
        ((Button) mLayout.findViewById(R.id.btn_barcode_print)).
                setOnClickListener(mPrintListener);
        ((Button) mLayout.findViewById(R.id.btn_barcode_close)).
                setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismiss();
            }
        });
    }

    @Override
    public String onRequestSend() throws PrinterException {
        mPrinterManager.setStandardModeAlignment(Alignment.ALIGN_CENTER);
        mPrinterManager.printBarcode(mType, mData, mModuleWidth,
                mModuleHeight, mHriPosition, mNwRatio);
        return getString(R.string.print_ok_msg);
    }

    private String getData() {
        return mEditTextData.getText().toString();
    }

    private int getModuleHeight() throws NumberFormatException {
        return Integer.parseInt(mEditTextModuleHeight.getText().toString());
    }

    private NwRatioBarcode getNwRatio() {
        RadioButton nwRatio1_2 =
                (RadioButton) mLayout.findViewById(R.id.rb_barcode_nw_ratio_1_2);
        RadioButton nwRatio1_25 =
                (RadioButton) mLayout.findViewById(R.id.rb_barcode_nw_ratio_1_25);
        RadioButton nwRatio1_3 =
                (RadioButton) mLayout.findViewById(R.id.rb_barcode_nw_ratio_1_3);

        NwRatioBarcode ret;
        if (nwRatio1_2.isChecked()) {
            ret = NwRatioBarcode.NWRATIO_BARCODE_1TO2;
        } else if (nwRatio1_25.isChecked()) {
            ret = NwRatioBarcode.NWRATIO_BARCODE_1TO2_5;
        } else if (nwRatio1_3.isChecked()) {
            ret = NwRatioBarcode.NWRATIO_BARCODE_1TO3;
        } else {
            ret = NwRatioBarcode.NWRATIO_BARCODE_1TO2;
        }
        return ret;
    }
}
