package com.seikoinstruments.sdk.mobileprinter.sample;

import android.app.Dialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.support.v4.content.ContextCompat;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import com.seikoinstruments.sdk.mobileprinter.CallbackFunctionListener;
import com.seikoinstruments.sdk.mobileprinter.PrinterException;
import com.seikoinstruments.sdk.mobileprinter.PrinterInterface;
import com.seikoinstruments.sdk.mobileprinter.PrinterModel;
import com.seikoinstruments.sdk.mobileprinter.PrinterStatus;

public class CallbackDialogFragment extends CommandDialogFragment {

    private Button mButtonOffline;
    private TextView mTextViewOffline;
    private TextView mTextViewVoltageError;
    private TextView mTextViewHardwareError;
    private TextView mTextViewHeadTemperatureError;
    private TextView mTextViewOutOfPaperError;
    private TextView mTextViewPaperJamError;
    private TextView mTextViewCoverOpenError;
    private TextView mTextViewBatteryError;
    private TextView mTextViewFeedSwitchState;
    private TextView mTextViewPaperFeedState;
    private TextView mTextViewReturnWaitingState;
    private TextView mTextViewFlashMemoryRewriting;
    private TextView mTextViewBatteryState;

    private View.OnClickListener mCloseConnectionListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            closePrinterManager();
        }
    };

    private CallbackFunctionListener mStateCallbackListener = new CallbackFunctionListener() {
        @Override
        public void onStatusChanged(PrinterStatus printerStatus) {
            setTextOfflineOrOnline(printerStatus.getErrOffline(), mButtonOffline,
                    mTextViewOffline);
            if(printerStatus.getErrOffline()){
                //clear
                clearAllStatusView();
                return;
            }
            setTextNoOrErr(printerStatus.getErrVoltage(), mTextViewVoltageError);
            setTextNoOrErr(printerStatus.getErrHardware(), mTextViewHardwareError);
            setTextNoOrErr(printerStatus.getErrHeadTemperature(),
                    mTextViewHeadTemperatureError);
            setTextNoOrErr(printerStatus.getErrOutOfPaper(), mTextViewOutOfPaperError);
            setTextNoOrErr(printerStatus.getErrMarkPaperJam(), mTextViewPaperJamError);
            setTextNoOrErr(printerStatus.getErrCoverOpen(), mTextViewCoverOpenError);
            setTextNoOrErr(printerStatus.getErrBattery(), mTextViewBatteryError);
            setTextOffOrOn(printerStatus.getStateFeedSwitch(), mTextViewFeedSwitchState);
            setTextStopOrFeeding(printerStatus.getStatePaperFeed(), mTextViewPaperFeedState);
            setTextNoOrErr(printerStatus.getStateReturnWaiting(), mTextViewReturnWaitingState);
            setTextNoOrErr(printerStatus.getStateFlashMemoryRewriting(),
                    mTextViewFlashMemoryRewriting);
            setTextBatteryLevel(printerStatus.getStateBattery(), mTextViewBatteryState);
        }
    };

    public void clearAllStatusView() {
        mTextViewVoltageError.setTextColor(getColor(R.color.callback_normal));
        mTextViewVoltageError.setText(getString(R.string.text_view_none));
        mTextViewHardwareError.setTextColor(getColor(R.color.callback_normal));
        mTextViewHardwareError.setText(getString(R.string.text_view_none));
        mTextViewHeadTemperatureError.setTextColor(getColor(R.color.callback_normal));
        mTextViewHeadTemperatureError.setText(getString(R.string.text_view_none));
        mTextViewOutOfPaperError.setTextColor(getColor(R.color.callback_normal));
        mTextViewOutOfPaperError.setText(getString(R.string.text_view_none));
        mTextViewPaperJamError.setTextColor(getColor(R.color.callback_normal));
        mTextViewPaperJamError.setText(getString(R.string.text_view_none));
        mTextViewCoverOpenError.setTextColor(getColor(R.color.callback_normal));
        mTextViewCoverOpenError.setText(getString(R.string.text_view_none));
        mTextViewBatteryError.setTextColor(getColor(R.color.callback_normal));
        mTextViewBatteryError.setText(getString(R.string.text_view_none));
        mTextViewFeedSwitchState.setTextColor(getColor(R.color.callback_normal));
        mTextViewFeedSwitchState.setText(getString(R.string.text_view_none));
        mTextViewPaperFeedState.setTextColor(getColor(R.color.callback_normal));
        mTextViewPaperFeedState.setText(getString(R.string.text_view_none));
        mTextViewReturnWaitingState.setTextColor(getColor(R.color.callback_normal));
        mTextViewReturnWaitingState.setText(getString(R.string.text_view_none));
        mTextViewFlashMemoryRewriting.setTextColor(getColor(R.color.callback_normal));
        mTextViewFlashMemoryRewriting.setText(getString(R.string.text_view_none));
        mTextViewBatteryState.setTextColor(getColor(R.color.callback_normal));
        mTextViewBatteryState.setText(getString(R.string.text_view_none));
    }

    public static CallbackDialogFragment newInstance(ConnectionInformation info) {
        CallbackDialogFragment fragment = new CallbackDialogFragment();
        Bundle args = new Bundle();
        args.putSerializable(MainActivity.CONN_INFO, info);
        fragment.setArguments(args);

        return fragment;
    }

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        setLayout(getActivity().getLayoutInflater().inflate(R.layout.callback_dialog, null));
        return super.onCreateDialog(savedInstanceState);
    }

    @Override
    public void onCancel(DialogInterface dialog) {
        super.onCancel(dialog);
        closePrinterManager();
    }

    @Override
    public void onStart() {
        super.onStart();
        if (!mPrinterManager.isOpened()) {
            openPrinterManager();
        }
    }

    @Override
    protected void setupWidgets() {
        // Change its color
        mButtonOffline = (Button) mLayout.findViewById(R.id.btn_callback_offline_color);

        mTextViewOffline = (TextView) mLayout.findViewById(R.id.tv_callback_offline_status);
        mTextViewVoltageError = (TextView) mLayout.findViewById(R.id.tv_voltage_error);
        mTextViewHardwareError = (TextView) mLayout.findViewById(R.id.tv_hardware_error);
        mTextViewHeadTemperatureError =
                (TextView) mLayout.findViewById(R.id.tv_head_temperature_error);
        mTextViewOutOfPaperError =
                (TextView) mLayout.findViewById(R.id.tv_out_of_paper_error);
        mTextViewPaperJamError = (TextView) mLayout.findViewById(R.id.tv_paper_jam_error);
        mTextViewCoverOpenError = (TextView) mLayout.findViewById(R.id.tv_cover_open_error);
        mTextViewBatteryError = (TextView) mLayout.findViewById(R.id.tv_battery_error);
        mTextViewFeedSwitchState =
                (TextView) mLayout.findViewById(R.id.tv_feed_switch_state);
        mTextViewPaperFeedState = (TextView) mLayout.findViewById(R.id.tv_paper_feed_state);
        mTextViewReturnWaitingState =
                (TextView) mLayout.findViewById(R.id.tv_return_waiting_state);
        mTextViewFlashMemoryRewriting =
                (TextView) mLayout.findViewById(R.id.tv_flash_memory_rewriting);
        mTextViewBatteryState = (TextView) mLayout.findViewById(R.id.tv_battery_state);

        Button btnClose = (Button) mLayout.findViewById(R.id.btn_callback_close);
        btnClose.setOnClickListener(mCloseConnectionListener);
    }

    private void openPrinterManager() {
        final MainActivity activity = (MainActivity) getActivity();
        activity.createProgressDialog(getString(R.string.dialog_message)).show();
        final PrinterInterface itf = getPrinterIf(mInfo);
        // Not allow to do network access on main thread
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    applySettingWithCurrentProperty();
                    mSocketKeepingTime = getPropertyInt(KEY_SOCKET_KEEPING_TIME);
                    mBtSecure = getPropertyBool(KEY_BT_SECURE);
                    mPrinterManager.open(itf, PrinterModel.PRN_MODEL_MP_A40,
                            mInfo.getAddress(), mSocketKeepingTime, mBtSecure);
                    if (mPrinterManager.isOpened()) {
                        mPrinterManager.startCallbackFunction(mStateCallbackListener);
                    }
                } catch (PrinterException e) {
                    popPrinterExceptionDialog(e.getErrorCode());
                    try {
                        if (mPrinterManager.isOpened()) {
                            mPrinterManager.close();
                        }
                    } catch (PrinterException e2) {
                        popPrinterExceptionDialog(e2.getErrorCode());
                    }
                    mPrinterManager = null;
                } finally {
                    activity.mHandler.post(new Runnable() {
                        @Override
                        public void run() {
                            activity.closeDialog();
                        }
                    });
                }
            }
        }, "Register callback").start();
    }

    private void closePrinterManager() {
        final MainActivity activity = (MainActivity) getActivity();
        activity.createProgressDialog(getString(R.string.dialog_message)).show();
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    if (mPrinterManager != null && mPrinterManager.isOpened()) {
                        mPrinterManager.stopCallbackFunction();
                    }
                } catch (PrinterException e) {
                    popPrinterExceptionDialog(e.getErrorCode());
                } finally {
                    try {
                        if (mPrinterManager != null && mPrinterManager.isOpened()) {
                            mPrinterManager.close();
                        }
                    } catch (PrinterException e) {
                        popPrinterExceptionDialog(e.getErrorCode());
                    }
                    mPrinterManager = null;
                    activity.mHandler.post(new Runnable() {
                        @Override
                        public void run() {
                            activity.closeDialog();
                            dismiss();
                        }
                    });
                }
            }
        }, "Unregister callback").start();
    }

    private void setTextOfflineOrOnline(boolean val, Button b, TextView v) {
        if (val) {
            b.setBackgroundColor(getColor(R.color.red));
            v.setText(getString(R.string.text_view_offline));
        } else {
            b.setBackgroundColor(getColor(R.color.green));
            v.setText(getString(R.string.text_view_online));
        }
    }

    private void setTextNoOrErr(boolean val, TextView v) {
        if (val) {
            v.setTextColor(getColor(R.color.callback_abnormal));
            v.setText(getString(R.string.text_view_err));
        } else {
            v.setTextColor(getColor(R.color.callback_normal));
            v.setText(getString(R.string.text_view_no));
        }
    }

    private void setTextOffOrOn(boolean val, TextView v) {
        if (val) {
            v.setTextColor(getColor(R.color.callback_abnormal));
            v.setText(getString(R.string.text_view_on));
        } else {
            v.setTextColor(getColor(R.color.callback_normal));
            v.setText(getString(R.string.text_view_off));
        }
    }

    private void setTextStopOrFeeding(boolean val, TextView v) {
        if (val) {
            v.setTextColor(getColor(R.color.callback_abnormal));
            v.setText(getString(R.string.text_view_feeding));
        } else {
            v.setTextColor(getColor(R.color.callback_normal));
            v.setText(getString(R.string.text_view_stop));
        }
    }

    private void setTextBatteryLevel(int n, TextView v) {
        int color = getColor(R.color.callback_normal);
        if( n == 0){
            v.setText("NONE");
        } else if( n == 1 ){
            v.setText("Level4");
        } else if( n == 2 ){
            v.setText("Level3");
        } else if( n == 3 ){
            v.setText("Level2");
        } else if( n == 4 ){
            v.setText("Level1");
        } else {
            color = getColor(R.color.callback_abnormal);
            v.setText("Err");
        }
        v.setTextColor(color);
    }

    private int getColor(int id) {
        return ContextCompat.getColor(getContext(), id);
    }
}
