package com.seikoinstruments.sdk.mobileprinter.sample;

import android.app.Dialog;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;

import com.seikoinstruments.sdk.mobileprinter.LogFileSize;
import com.seikoinstruments.sdk.mobileprinter.PrinterException;

public class LogSettingDialogFragment extends CommandDialogFragment {

    private static final int LOG_LEVEL = 0;

    private LogFileSize mFileSize;

    private static final LogFileSize[] sTypeFileSize = new LogFileSize[] {
            LogFileSize.LOG_FILE_SIZE_1MB,
            LogFileSize.LOG_FILE_SIZE_5MB,
            LogFileSize.LOG_FILE_SIZE_10MB,
            LogFileSize.LOG_FILE_SIZE_50MB
    };

    private AdapterView.OnItemSelectedListener mFileSizeListener =
            new AdapterView.OnItemSelectedListener() {
        @Override
        public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
            mFileSize = sTypeFileSize[position];
        }

        @Override
        public void onNothingSelected(AdapterView<?> parent) {
            mFileSize = sTypeFileSize[0];
        }
    };

    private View.OnClickListener mApplyListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            applySetting();
        }
    };

    public static LogSettingDialogFragment newInstance(ConnectionInformation info) {
        LogSettingDialogFragment fragment = new LogSettingDialogFragment();
        Bundle args = new Bundle();
        args.putSerializable(MainActivity.CONN_INFO, info);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        setLayout(getActivity().getLayoutInflater().inflate(R.layout.log_setting_dialog, null));
        return super.onCreateDialog(savedInstanceState);
    }

    @Override
    protected void setupWidgets() {
        // Log File Size
        Spinner spFileSize = (Spinner) mLayout.findViewById(R.id.sp_log_file_size);
        ArrayAdapter<CharSequence> fileSizeAdapter = ArrayAdapter.createFromResource(
                getActivity(), R.array.log_file_size, android.R.layout.simple_spinner_item);
        fileSizeAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spFileSize.setAdapter(fileSizeAdapter);
        spFileSize.setOnItemSelectedListener(mFileSizeListener);

        // print and cancel button
        ((Button) mLayout.findViewById(R.id.btn_log_apply)).setOnClickListener(mApplyListener);
        ((Button) mLayout.findViewById(R.id.btn_log_cancel)).
                setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismiss();
            }
        });
    }

    private void applySetting() {
        try {
            mPrinterManager.setLog(LOG_LEVEL, mFileSize);
            popInfoDialog(getString(R.string.setting_ok_msg));
        } catch (PrinterException e) {
            popPrinterExceptionDialog(e.getErrorCode());
        }
    }
}
