package com.seikoinstruments.sdk.mobileprinter.sample;

import android.Manifest;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.bluetooth.BluetoothAdapter;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentSender;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.provider.MediaStore;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.content.PermissionChecker;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.Toast;

import com.google.android.gms.common.api.ApiException;
import com.google.android.gms.common.api.ResolvableApiException;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.location.LocationSettingsRequest;
import com.google.android.gms.location.LocationSettingsResponse;
import com.google.android.gms.location.LocationSettingsStatusCodes;
import com.google.android.gms.location.SettingsClient;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.seikoinstruments.sdk.mobileprinter.Alignment;
import com.seikoinstruments.sdk.mobileprinter.Bold;
import com.seikoinstruments.sdk.mobileprinter.CharacterScale;
import com.seikoinstruments.sdk.mobileprinter.CharacterSet;
import com.seikoinstruments.sdk.mobileprinter.CharacterType;
import com.seikoinstruments.sdk.mobileprinter.Direction;
import com.seikoinstruments.sdk.mobileprinter.Dithering;
import com.seikoinstruments.sdk.mobileprinter.ErrorCorrect2Dcode;
import com.seikoinstruments.sdk.mobileprinter.HriPositionBarcode;
import com.seikoinstruments.sdk.mobileprinter.InternationalCharacterSet;
import com.seikoinstruments.sdk.mobileprinter.Mode2Dcode;
import com.seikoinstruments.sdk.mobileprinter.ModuleSize2Dcode;
import com.seikoinstruments.sdk.mobileprinter.ModuleWidthBarcode;
import com.seikoinstruments.sdk.mobileprinter.NwRatioBarcode;
import com.seikoinstruments.sdk.mobileprinter.PrinterException;
import com.seikoinstruments.sdk.mobileprinter.PrinterInterface;
import com.seikoinstruments.sdk.mobileprinter.PrinterManager;
import com.seikoinstruments.sdk.mobileprinter.PrinterModel;
import com.seikoinstruments.sdk.mobileprinter.Reverse;
import com.seikoinstruments.sdk.mobileprinter.Rotate;
import com.seikoinstruments.sdk.mobileprinter.TransactionFunction;
import com.seikoinstruments.sdk.mobileprinter.Type2DCode;
import com.seikoinstruments.sdk.mobileprinter.TypeBarcode;
import com.seikoinstruments.sdk.mobileprinter.Underline;

public class MainActivity extends FragmentActivity
        implements ScannerDialogFragment.NoticeDialogListener {

    public static final String CONN_INFO =
            "com.seikoinstruments.sdk.thermalprinter.siiandroidsdksample.connection_information";
    private static final int STANDARD_MODE = 0;
    private static final int PAGE_MODE = 1;

    private static final int REQUEST_CODE_BLUETOOTH = 0;
    private static final int REQUEST_CODE_SENDFILE = 1;
    private static final int REQUEST_ENABLE_LOCATION = 2;

    private EditText mEditTextAddress;
    private Button mButtonList;
    protected final Handler mHandler = new Handler();
    public static PrinterManager sPrinterManager;
    protected Dithering mDithering = Dithering.DITHERING_DISABLE;
    protected String  mFilePath = "";
    private static final int DEFAULT_INDEX = 1;
    private int checkedItem = DEFAULT_INDEX;

    private ProgressDialog mProgressDialog;

    private View.OnClickListener mInterfaceListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            if (((RadioButton) v).isChecked()) {
                switch (v.getId()) {
                    case R.id.rb_interface_bt:
                        mEditTextAddress.setVisibility(View.VISIBLE);
                        mButtonList.setVisibility(View.VISIBLE);
                        break;
                    case R.id.rb_interface_usb:
                        mEditTextAddress.setVisibility(View.INVISIBLE);
                        mButtonList.setVisibility(View.INVISIBLE);
                        break;
                    case R.id.rb_interface_tcpip:
                        mEditTextAddress.setVisibility(View.VISIBLE);
                        mButtonList.setVisibility(View.VISIBLE);
                        break;
                }
            }
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        sPrinterManager = new PrinterManager(getApplicationContext());
        setupWidgets();
        initSetting();
    }

    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        switch(requestCode) {
            case REQUEST_ENABLE_LOCATION:
                discoverDevices();
                break;
            case REQUEST_CODE_SENDFILE:
                if (resultCode == RESULT_OK) {
                    Uri uri = null;
                    if (data != null) {
                        uri = data.getData();
                        mFilePath = uri.toString();
                        sendDataFile();
                    }
                }
                break;
            default:
                break;
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if(grantResults[0] == PackageManager.PERMISSION_GRANTED){
            if(requestCode == REQUEST_CODE_BLUETOOTH) {
                //for Android 12
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                    if ((PermissionChecker.checkSelfPermission(MainActivity.this, Manifest.permission.BLUETOOTH_CONNECT) != PackageManager.PERMISSION_GRANTED)
                            || (PermissionChecker.checkSelfPermission(MainActivity.this, Manifest.permission.BLUETOOTH_SCAN) != PackageManager.PERMISSION_GRANTED)) {
                        ActivityCompat.requestPermissions(MainActivity.this, new String[]{Manifest.permission.BLUETOOTH_CONNECT, Manifest.permission.BLUETOOTH_SCAN}, REQUEST_CODE_BLUETOOTH);
                        return;
                    }
                }
                //for Android 11
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
                    checkLocation();
                }else {
                    discoverDevices();
                }
            }
        }
    }

    @Override
    public void onAddressClick(String address) {
        mEditTextAddress.setText(address);
    }

    private void setupWidgets() {
        RadioButton rbBluetooth = (RadioButton) findViewById(R.id.rb_interface_bt);
        rbBluetooth.setOnClickListener(mInterfaceListener);
        RadioButton rbUsb = (RadioButton) findViewById(R.id.rb_interface_usb);
        rbUsb.setOnClickListener(mInterfaceListener);
        RadioButton rbTcpip = (RadioButton) findViewById(R.id.rb_interface_tcpip);
        rbTcpip.setOnClickListener(mInterfaceListener);
        mEditTextAddress = (EditText) findViewById(R.id.et_address);
        mButtonList = (Button) findViewById(R.id.btn_list);
    }

    public void onScanClicked(View v) {
        // Pop activity dialog
        if(getPrinterInterface() == PrinterInterface.PRN_IF_BT){
            if(PermissionChecker.checkSelfPermission(this,Manifest.permission.ACCESS_FINE_LOCATION)!=PackageManager.PERMISSION_GRANTED){
                ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, REQUEST_CODE_BLUETOOTH);
                return;
            }
            //for Android 12
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                if((PermissionChecker.checkSelfPermission(this,Manifest.permission.BLUETOOTH_CONNECT)!=PackageManager.PERMISSION_GRANTED)
                        || (PermissionChecker.checkSelfPermission(this,Manifest.permission.BLUETOOTH_SCAN)!=PackageManager.PERMISSION_GRANTED)){
                    ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.BLUETOOTH_CONNECT, Manifest.permission.BLUETOOTH_SCAN}, REQUEST_CODE_BLUETOOTH);
                    return;
                }
            }
            //for Android 11
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
                checkLocation();
                return;
            }
        }
        discoverDevices();
    }

    private void discoverDevices() {
        DialogFragment dialog = ScannerDialogFragment.newInstance(getString(R.string.scan_title), getConnectionInformation());
        FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
        ft.add(dialog, "Scan");
        ft.commitAllowingStateLoss();
    }

    private void checkLocation() {
        BluetoothAdapter bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        if (bluetoothAdapter.isEnabled() == true) {
            final LocationRequest locationRequest = LocationRequest.create();
            locationRequest.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);
            LocationSettingsRequest.Builder builder = new LocationSettingsRequest.Builder().addLocationRequest(locationRequest);
            SettingsClient client = LocationServices.getSettingsClient(this);
            Task<LocationSettingsResponse> task = client.checkLocationSettings(builder.build());
            task.addOnCompleteListener(new OnCompleteListener<LocationSettingsResponse>() {
                @Override
                public void onComplete(@NonNull Task<LocationSettingsResponse> task) {
                    try {
                        task.getResult(ApiException.class);
                        discoverDevices();
                    } catch (ApiException exception) {
                        switch (exception.getStatusCode()) {
                            case LocationSettingsStatusCodes.RESOLUTION_REQUIRED:
                                try {
                                    ResolvableApiException resolvable = (ResolvableApiException) exception;
                                    resolvable.startResolutionForResult(MainActivity.this, REQUEST_ENABLE_LOCATION);
                                    return;
                                } catch (IntentSender.SendIntentException e) {
                                } catch (ClassCastException e) {
                                }
                                break;
                            case LocationSettingsStatusCodes.SETTINGS_CHANGE_UNAVAILABLE:
                                break;
                        }
                    }
                }
            });
        }
    }

    public void onRegisterCallbackClicked(View v) {
        openSelections(CallbackDialogFragment.class,
                R.layout.callback_dialog, "Register Callback");
    }

    public void onPrinterInformationClicked(View v) {
        openSelections(PrinterInformationDialogFragment.class,
                R.layout.printer_information_dialog, "Printer Information");
    }

    public void onStandardModeSampleClicked(View v) {
        printSampleReceipt(STANDARD_MODE);
    }

    public void onPageModeSampleClicked(View v) {
        printSampleReceipt(PAGE_MODE);
    }

    public void onSendFileClicked(View v) {
        openFileSelections();
    }

    public void onLogFileClicked(View v) {
        openSelections(LogSettingDialogFragment.class,
                R.layout.log_setting_dialog, "Log Setting");
    }

    public void onSettingClicked(View v) {
        openSelections(SettingDialogFragment.class,
                R.layout.setting_dialog, "Setting");
    }

    public void onTextFormattingClicked(View v) {
        openSelections(TextFormattingDialogFragment.class,
                R.layout.text_formatting_dialog, "Text Formatting");
    }

    public void onBarcodeClicked(View v) {
        openSelections(BarcodeDialogFragment.class,
                R.layout.barcode_dialog, "Barcode");
    }

    public void on2DBarcodeClicked(View v) {
        openSelections(TwoDimensionalBarcodeDialogFragment.class,
                R.layout.two_dimensional_barcode_dialog, "2DCode");
    }

    private void openSelections(Class<? extends CommandDialogFragment> cls,
                                int layoutId, String tag) {
        String clsName = cls.getSimpleName();
        DialogFragment dialog;
        switch (clsName) {
            case "PrinterInformationDialogFragment":
                dialog = PrinterInformationDialogFragment.newInstance(getConnectionInformation());
                break;
            case "TextFormattingDialogFragment":
                dialog = TextFormattingDialogFragment.newInstance(getConnectionInformation());
                break;
            case "BarcodeDialogFragment":
                dialog = BarcodeDialogFragment.newInstance(getConnectionInformation());
                break;
            case "TwoDimensionalBarcodeDialogFragment":
                dialog = TwoDimensionalBarcodeDialogFragment.newInstance(getConnectionInformation());
                break;
            case "CallbackDialogFragment":
                dialog = CallbackDialogFragment.newInstance(getConnectionInformation());
                break;
            case "LogSettingDialogFragment":
                dialog = LogSettingDialogFragment.newInstance(getConnectionInformation());
                break;
            case "SettingDialogFragment":
                dialog = SettingDialogFragment.newInstance(getConnectionInformation());
                break;
            default:
                dialog = new CommandDialogFragment();
                break;
        }
        FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
        ft.add(dialog, tag);
        ft.commitAllowingStateLoss();
    }

    private ConnectionInformation getConnectionInformation() {
        return new ConnectionInformation(getPrinterInterface().getIntValue(),
                mEditTextAddress.getText().toString());
    }

    private void printSampleReceipt(final int mode) {
        createProgressDialog(getString(R.string.dialog_message)).show();
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    int socketKeepingTime = getPropertyInt(CommandDialogFragment.KEY_SOCKET_KEEPING_TIME);
                    int writeTimeout = getPropertyInt(CommandDialogFragment.KEY_WRITE_TIMEOUT);
                    int responseTimeout = getPropertyInt(CommandDialogFragment.KEY_RESPONSE_TIMEOUT);
                    boolean btSecure = getPropertyBool(CommandDialogFragment.KEY_BT_SECURE);

                    sPrinterManager.setWriteTimeout(writeTimeout);
                    sPrinterManager.setResponseTimeout(responseTimeout);

                    sPrinterManager.open(getPrinterInterface(), PrinterModel.PRN_MODEL_MP_A40,
                            mEditTextAddress.getText().toString(), socketKeepingTime, btSecure);
                    sendSampleReceiptCmd(sPrinterManager, mode);
                    popSuccessDialog();
                } catch (PrinterException e) {
                    popPrinterExceptionDialog(e);
                } finally {
                    if (sPrinterManager != null && sPrinterManager.isOpened()) {
                        try {
                            sPrinterManager.close();
                        } catch (PrinterException e) {
                            popPrinterExceptionDialog(e);
                        }
                    }
                    mHandler.post(new Runnable() {
                        @Override
                        public void run() {
                            closeDialog();
                        }
                    });
                }
            }
        }, "printSampleReceipt").start();
    }

    private void initSetting() {
        CommandDialogFragment fm = getFragment();
        fm.setLocaleProperty(sPrinterManager.getCharacterSet(),
                sPrinterManager.getInternationalCharacterSet());
    }

    private int getPropertyInt(String key) {
        CommandDialogFragment fm = getFragment();
        return fm.getPropertyInt(key);
    }

    private boolean getPropertyBool(String key) {
        CommandDialogFragment fm = getFragment();
        return fm.getPropertyBool(key);
    }

    private void popSuccessDialog() {
        CommandDialogFragment fm = getFragment();
        fm.popInfoDialog(getString(R.string.print_ok_msg));
    }

    private void popPrinterExceptionDialog(PrinterException exception) {
        CommandDialogFragment fm = getFragment();
        fm.popPrinterExceptionDialog(exception.getErrorCode());
    }

    private CommandDialogFragment getFragment() {
        CommandDialogFragment fm = new CommandDialogFragment();
        fm.setActivity(this);
        return fm;
    }

    private PrinterInterface getPrinterInterface() {
        PrinterInterface itf;
        if (((RadioButton) findViewById(R.id.rb_interface_bt)).isChecked()) {
            itf = PrinterInterface.PRN_IF_BT;
        } else if (((RadioButton) findViewById(R.id.rb_interface_usb)).isChecked()) {
            itf = PrinterInterface.PRN_IF_USB;
        } else if (((RadioButton) findViewById(R.id.rb_interface_tcpip)).isChecked()) {
            itf = PrinterInterface.PRN_IF_TCP;
        } else {
            itf = PrinterInterface.PRN_IF_BT;
        }
        return itf;
    }

    private void sendSampleReceiptCmd(PrinterManager printerManager, int printMode)
            throws PrinterException {
        //===== define receipt property start =====
        final int WRITE_TIMEOUT = 15000;
        final int STD_MODE_LEFT_MARGIN = 0;
        final int STD_MODE_PRINT_AREA_WIDTH = 832;
        final int WIDTH_RECEIPT = 576;
        final int HEIGHT_RECEIPT = 1400;
        final int WIDTH_FONT_A = 12;
        final int HEIGHT_FONT_A = 24;
        int startX = 0, startY = 0;
        int width = WIDTH_RECEIPT, height = HEIGHT_RECEIPT;
        final int LINE_SPACE = 34;
        final int RIGHT_SPACE = 0;
        final int THICKNESS = 2;
        final int HORIZONTAL_POS = 0;
        final int HEIGHT_BARCODE = 162;
        final int HEIGHT_2DCODE = 10;
        final int COLUM_2DCODE = 0;
        final int ROW_2DCODE = 0;
        String receiptTitle = "Receipt Sample\n\n";
        String receiptTime = "No.123456789     03/04/11  12:49 PM\n";
        String receiptContent =
                "--------------------------------\n" +
                        "GRILLED CHICKEN BREAST   $ 18.50\n" +
                        "SIRLOIN STEAK            $ 32.00\n" +
                        "ROAST LAMB               $ 20.00\n" +
                        "SALAD                    $ 10.00\n" +
                        "COKE                     $  3.50\n" +
                        "COKE                     $  3.50\n" +
                        "ICE CREAM                $  5.00\n" +
                        "CHINESE NOODLE           $ 15.00\n" +
                        "SUKIYAKI                 $ 30.00\n" +
                        "SANDWICH                 $ 10.00\n" +
                        "PIZZA                    $ 20.00\n" +
                        "TEA                      $  3.50\n" +
                        "COFFEE                   $  3.50\n\n" +
                        "--------------------------------\n" +
                        "        SUBTOTAL        $ 174.50\n" +
                        "        SALES TAX       $   8.73\n" +
                        "        TOTAL           $ 183.23\n\n" +
                        "Thank you and see you again!\n";
        String receiptBarcode = "12345678905";
        String receipt2DCode = "ABC0123";
        //===== define receipt property end =====

        if (printMode != STANDARD_MODE && printMode != PAGE_MODE) {
            printMode = STANDARD_MODE;
        }

        printerManager.setWriteTimeout(WRITE_TIMEOUT);
        printerManager.selectCharacterSet(CharacterSet.CODEPAGE_437);
        printerManager.
                selectInternationalCharacterSet(InternationalCharacterSet.INT_CHAR_SET_USA);
        //===== Start transaction =====
        printerManager.controlTransaction(TransactionFunction.TRANSACTION_START);

        //===== Total area =====
        switch (printMode) {
            case STANDARD_MODE:
                printerManager.selectStandardMode();
                printerManager.setStandardModeArea(STD_MODE_LEFT_MARGIN,
                        STD_MODE_PRINT_AREA_WIDTH);
                printerManager.setStandardModeAlignment(Alignment.ALIGN_CENTER);
                break;
            case PAGE_MODE:
                printerManager.selectPageMode(startX,
                        startY,
                        WIDTH_RECEIPT,
                        HEIGHT_RECEIPT,
                        Direction.DIRECTION_LEFT_TO_RIGHT);
                break;
            default:
                break;
        }

        printerManager.setLineSpacing(LINE_SPACE);
        printerManager.setCharacterRightSpace(RIGHT_SPACE);

        //===== Title =====
        switch (printMode) {
            case STANDARD_MODE:
                //set title property for Standard Mode
                receiptTitle = "Receipt Sample\n\n";

                printerManager.setHorizontalPosition(HORIZONTAL_POS);
                break;
            case PAGE_MODE:
                //print rectangle for Page Mode
                printerManager.selectPageMode(startX,
                        startY,
                        WIDTH_RECEIPT,
                        HEIGHT_RECEIPT,
                        Direction.DIRECTION_LEFT_TO_RIGHT);
                printerManager.printPageModeRectangle(startX,
                        startY,
                        WIDTH_RECEIPT,
                        HEIGHT_RECEIPT,
                        THICKNESS);

                //set title property for Page Mode
                receiptTitle = "Receipt Sample";

                int widthReceiptTitle = ((WIDTH_FONT_A + RIGHT_SPACE) * 2) * receiptTitle.length();
                startX = (WIDTH_RECEIPT - widthReceiptTitle) / 2;
                startY += 80;
                width = WIDTH_RECEIPT - startX;
                height = HEIGHT_FONT_A * 1;

                printerManager.selectPageMode(startX,
                        startY,
                        width,
                        height,
                        Direction.DIRECTION_LEFT_TO_RIGHT);
                printerManager.setPageModeVerticalPosition(HEIGHT_FONT_A);
                break;
            default:
                break;
        }

        printerManager.setCharacterFormatting(CharacterType.CHAR_TYPE_FONT_A,
                CharacterScale.CHAR_SCALE_X1,
                CharacterScale.CHAR_SCALE_X2,
                Underline.CHAR_UNDERLINE_NONE,
                Bold.CHAR_BOLD_ON,
                Reverse.CHAR_REVERSE_OFF,
                Rotate.ROTATE_NONE);

        //print title
        printerManager.printText(receiptTitle);

        //===== No, and Time =====
        switch (printMode) {
            case STANDARD_MODE:
                //set time property for Standard Mode
                receiptTime = "No.123456789     03/04/11  12:49 PM\n";

                printerManager.setHorizontalPosition(HORIZONTAL_POS);
                printerManager.setCharacterFormatting(CharacterType.CHAR_TYPE_FONT_A,
                        CharacterScale.CHAR_SCALE_X1,
                        CharacterScale.CHAR_SCALE_X1,
                        Underline.CHAR_UNDERLINE_NONE,
                        Bold.CHAR_BOLD_OFF,
                        Reverse.CHAR_REVERSE_OFF,
                        Rotate.ROTATE_NONE);
                //do not print time here
                break;
            case PAGE_MODE:
                //set time property for Page Mode
                receiptTime = "No.123456789     03/04/11  12:49 PM";
                startX = (WIDTH_RECEIPT - ((WIDTH_FONT_A + RIGHT_SPACE) * receiptTime.length())) / 2;
                startY += height + 80;
                width = WIDTH_RECEIPT - startX;
                height = HEIGHT_FONT_A * 1;

                printerManager.selectPageMode(startX,
                        startY,
                        width,
                        height,
                        Direction.DIRECTION_LEFT_TO_RIGHT);
                printerManager.setPageModeVerticalPosition(HEIGHT_FONT_A);
                printerManager.setCharacterFormatting(CharacterType.CHAR_TYPE_FONT_A,
                        CharacterScale.CHAR_SCALE_X1,
                        CharacterScale.CHAR_SCALE_X1,
                        Underline.CHAR_UNDERLINE_NONE,
                        Bold.CHAR_BOLD_OFF,
                        Reverse.CHAR_REVERSE_OFF,
                        Rotate.ROTATE_NONE);

                //print time for Page Mode
                printerManager.printText(receiptTime);
                break;
            default:
                break;
        }

        //===== Content =====
        switch (printMode) {
            case STANDARD_MODE:
                //set content property for Standard Mode
                printerManager.setHorizontalPosition(HORIZONTAL_POS);

                //print time and content
                printerManager.printText(receiptTime + receiptContent);
                printerManager.feedLine(2);
                break;
            case PAGE_MODE:
                //set content property for Page Mode
                startX = (WIDTH_RECEIPT - ((WIDTH_FONT_A + RIGHT_SPACE) * 32)) / 2;
                startY += height + 40;
                width = WIDTH_RECEIPT - startX;
                height = (HEIGHT_FONT_A + (LINE_SPACE - HEIGHT_FONT_A)) * 21;

                printerManager.selectPageMode(startX,
                        startY,
                        width,
                        height,
                        Direction.DIRECTION_LEFT_TO_RIGHT);
                printerManager.setPageModeVerticalPosition(HEIGHT_FONT_A);

                //print content
                printerManager.printText(receiptContent);
                break;
            default:
                break;
        }

        //===== Barcode =====
        switch (printMode) {
            case STANDARD_MODE:
                //set barcode property for Standard Mode
                printerManager.setStandardModeBarcodeDirection(Rotate.ROTATE_NONE);
                printerManager.setHorizontalPosition(HORIZONTAL_POS);
                break;
            case PAGE_MODE:
                //set barcode property for Page Mode
                startX = 100;
                startY += height + 40;
                width = WIDTH_RECEIPT - startX;
                height = 200;

                printerManager.selectPageMode(startX,
                        startY,
                        width,
                        height,
                        Direction.DIRECTION_LEFT_TO_RIGHT);
                printerManager.setPageModeVerticalPosition(162);
                break;
            default:
                break;
        }

        //print barcode
        printerManager.printBarcode(TypeBarcode.TYPE_BARCODE_UPC_A,
                receiptBarcode,
                ModuleWidthBarcode.MODULE_WIDTH_BARCODE_3,
                HEIGHT_BARCODE,
                HriPositionBarcode.HRI_BARCODE_BOTTOM_FONT_A,
                NwRatioBarcode.NWRATIO_BARCODE_1TO2);

        //===== 2DCode =====
        switch (printMode) {
            case STANDARD_MODE:
                //set 2Dcode property for Standard Mode
                printerManager.setHorizontalPosition(HORIZONTAL_POS);
                break;
            case PAGE_MODE:
                //set 2Dcode property for Page Mode
                startX = 150;
                startY += height + 40;
                width = WIDTH_RECEIPT - startX;
                height = 100;

                printerManager.selectPageMode(startX,
                        startY,
                        width,
                        height,
                        Direction.DIRECTION_LEFT_TO_RIGHT);
                printerManager.setPageModeVerticalPosition(90);
                break;
            default:
                break;
        }

        //print 2Dcode
        printerManager.print2DCode(Type2DCode.TYPE_2DCODE_PDF417,
                receipt2DCode,
                Mode2Dcode.MODE_2DCODE_PDF417_STANDARD,
                ModuleSize2Dcode.MODULE_SIZE_2DCODE_3,
                HEIGHT_2DCODE,
                COLUM_2DCODE,
                ROW_2DCODE,
                ErrorCorrect2Dcode.ERR_CORRECTION_2DCODE_PDF417_0);

        if (printMode == PAGE_MODE) {
            printerManager.printPageModeData();
            printerManager.selectStandardMode();
        }

        //===== cut receipt =====
        printerManager.feedCutPosition();

        //===== Print transaction
        printerManager.controlTransaction(TransactionFunction.TRANSACTION_PRINT);
    }

    public Dialog createProgressDialog(String message) {
        if (mProgressDialog != null) {
            closeDialog();
        }
        mProgressDialog = new ProgressDialog(this);
        mProgressDialog.setTitle(R.string.app_name);
        mProgressDialog.setMessage(message);
        mProgressDialog.setIndeterminate(false);
        mProgressDialog.setCancelable(false);
        mProgressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        mProgressDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
            public void onCancel(DialogInterface dialog) {
                //No processing
            }
        });
        return mProgressDialog;
    }

    public void closeDialog() {
        try {
            mProgressDialog.dismiss();
        } catch (Exception e) {
        }
        mProgressDialog = null;
    }

    private void openFileSelections() {
        Intent intent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
        intent.addCategory(Intent.CATEGORY_OPENABLE);
        intent.setType("*/*");
        startActivityForResult(intent, REQUEST_CODE_SENDFILE);
    }

    private void sendDataFile() {
        String uriFileName = getUriFileName(mFilePath);
        String extension = getFileExtension(uriFileName);

        switch (extension) {
            case "bmp":
            case "jpg":
            case "jpeg":
            case "png":
                //Display Dithering setting dialog
                final String items[] = {"Disabled", "Enabled"};
                AlertDialog dialog = new AlertDialog.Builder(MainActivity.this)
                        .setTitle("Dithering")
                        .setSingleChoiceItems(items, DEFAULT_INDEX, new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                checkedItem = which;
                                Toast.makeText(MainActivity.this, items[which], Toast.LENGTH_SHORT).show();
                            }
                        })
                        .setNegativeButton("OK", new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                if (checkedItem == 0) {
                                    mDithering = Dithering.DITHERING_DISABLE;
                                } else {
                                    mDithering = Dithering.DITHERING_ERRORDIFFUSION;
                                }
                                dialog.dismiss();
                                doSendDataFile();
                            }
                        }).create();
                dialog.show();

                break;
            default:
                doSendDataFile();
        }
    }

    private void doSendDataFile() {
        createProgressDialog(getString(R.string.dialog_message)).show();
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    int socketKeepingTime = getPropertyInt(CommandDialogFragment.KEY_SOCKET_KEEPING_TIME);
                    boolean btSecure = getPropertyBool(CommandDialogFragment.KEY_BT_SECURE);
                    sPrinterManager.open(getPrinterInterface(), PrinterModel.PRN_MODEL_MP_A40,
                            mEditTextAddress.getText().toString(), socketKeepingTime, btSecure);
                    sPrinterManager.setStandardModeAlignment(Alignment.ALIGN_LEFT);
                    sPrinterManager.sendDataFile(mFilePath, mDithering);
                    popSuccessDialog();
                } catch (PrinterException e) {
                    popPrinterExceptionDialog(e);
                } finally {
                    if (sPrinterManager != null && sPrinterManager.isOpened()) {
                        try {
                            sPrinterManager.close();
                        } catch (PrinterException e) {
                            popPrinterExceptionDialog(e);
                        }
                    }
                    mHandler.post(new Runnable() {
                        @Override
                        public void run() {
                            closeDialog();
                        }
                    });
                }
            }
        }, "Opening").start();
    }

    private String getUriFileName(String path){
        String fileName = "";
        String[] projection = {MediaStore.MediaColumns.DISPLAY_NAME};
        Cursor cursor = getApplicationContext().getContentResolver()
                .query(Uri.parse(path), projection, null, null, null);
        if (cursor != null) {
            if (cursor.moveToFirst()) {
                fileName = cursor.getString(
                        cursor.getColumnIndexOrThrow(MediaStore.MediaColumns.DISPLAY_NAME));
            }
            cursor.close();
        }
        return fileName;
    }

    protected String getFileExtension(String name) {
        String extension;
        final int index = name.lastIndexOf(".");
        if (index != -1) {
            extension = name.substring(index + 1);
        } else {
            extension = name;
        }

        return extension.toLowerCase();
    }
}