package com.seikoinstruments.sdk.mobileprinter.sample;

import android.app.Dialog;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;

import com.seikoinstruments.sdk.mobileprinter.PrinterException;
import com.seikoinstruments.sdk.mobileprinter.PrinterInformation;

public class PrinterInformationDialogFragment extends CommandDialogFragment
                                    implements CommandDialogFragment.RequestSender {

    private static final int INFO_BYTES = 0x01;
    private static final int INFO_STRING = 0x02;
    private static final int INFO_NUMBER = 0x03;

    private int mType;
    private PrinterInformation mPrinterInformation;

    public static PrinterInformationDialogFragment newInstance(ConnectionInformation info) {
        PrinterInformationDialogFragment fragment = new PrinterInformationDialogFragment();
        Bundle args = new Bundle();
        args.putSerializable(MainActivity.CONN_INFO, info);
        fragment.setArguments(args);

        return fragment;
    }

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        mRequestSender = this;
        setLayout(getActivity().getLayoutInflater().
                inflate(R.layout.printer_information_dialog, null));
        return super.onCreateDialog(savedInstanceState);
    }

    @Override
    protected void setupWidgets() {

        Button btnNvMemSpace = (Button) mLayout.findViewById(R.id.pinfo_nv_mem_space);
        btnNvMemSpace.
                setOnClickListener(new NumCommandListener(PrinterInformation.GET_NV_MEM_CAP));
        Button btnNvMemRemainSpace =
                (Button) mLayout.findViewById(R.id.pinfo_nv_mem_remain_space);
        btnNvMemRemainSpace.
                setOnClickListener(new NumCommandListener(PrinterInformation.GET_NV_MEM_REM_CAP));
        ((Button) mLayout.findViewById(R.id.pinfo_nv_mem_keycode_list)).setOnClickListener(
                new StringCommandListener(PrinterInformation.GET_NV_MEM_KEYCODE_LIST)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_rem_user_mem_cap)).setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_REM_USER_MEM_CAP)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_rem_user_mem_cap_defrag)).setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_REM_USER_MEM_CAP_DEFRAG)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_func_set_resp)).setOnClickListener(
                new CommandListener(PrinterInformation.GET_FUNC_SET_RESP)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_prn_id_model)).setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_PRN_ID_MODEL)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_prn_id_type)).setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_PRN_ID_TYPE)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_prn_id_rom_ver)).setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_PRN_ID_ROM_VER)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_prn_id_firm_ver_main)).setOnClickListener(
                new StringCommandListener(PrinterInformation.GET_PRN_ID_FIRM_VER_MAIN)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_prn_id_manufacturer)).setOnClickListener(
                new StringCommandListener(PrinterInformation.GET_PRN_ID_MFR)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_prn_id_model_name)).setOnClickListener(
                new StringCommandListener(PrinterInformation.GET_PRN_ID_MODEL_NAME)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_prn_id_firm_ver_boot)).setOnClickListener(
                new StringCommandListener(PrinterInformation.GET_PRN_ID_FIRM_VER_BOOT)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_prn_id_firm_checksum_boot)).
                setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_PRN_ID_FIRM_CHECKSUM_BOOT)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_prn_id_firm_checksum_main)).
                setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_PRN_ID_FIRM_CHECKSUM_MAIN)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_prn_id_firm_checksum)).setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_PRN_ID_FIRM_CHECKSUM)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_maint_num_feed_line)).setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_MAINT_NUM_FEED_LINE)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_maint_num_head_active)).setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_MAINT_NUM_HEAD_ACTIVE)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_maint_drive_time)).setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_MAINT_DRIVE_TIME)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_maint_num_feed_line_integration)).
                setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_MAINT_NUM_FEED_LINE_INTEGRATION)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_maint_num_head_active_integration)).
                setOnClickListener(new NumCommandListener(
                                PrinterInformation.GET_MAINT_NUM_HEAD_ACTIVE_INTEGRATION)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_maint_drive_time_integration)).
                setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_MAINT_DRIVE_TIME_INTEGRATION)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_hfont_24_checksum)).setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_HFONT_24_CHECKSUM)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_hfont_24_id)).setOnClickListener(
                new StringCommandListener(PrinterInformation.GET_HFONT_24_ID)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_hfont_24_int_char)).setOnClickListener(
                new StringCommandListener(PrinterInformation.GET_HFONT_24_INT_CHAR)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_hfont_16_checksum)).setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_HFONT_16_CHECKSUM)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_hfont_16_id)).setOnClickListener(
                new StringCommandListener(PrinterInformation.GET_HFONT_16_ID)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_hfont_16_int_char)).setOnClickListener(
                new StringCommandListener(PrinterInformation.GET_HFONT_16_INT_CHAR)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_ffont_lang)).setOnClickListener(
                new StringCommandListener(PrinterInformation.GET_FFONT_LANG)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_ffont_standard)).setOnClickListener(
                new StringCommandListener(PrinterInformation.GET_FFONT_STANDARD)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_ffont_company)).setOnClickListener(
                new StringCommandListener(PrinterInformation.GET_FFONT_COMPANY)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_ffont_checksum)).setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_FFONT_CHECKSUM)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_ffont_data_size)).setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_FFONT_DATA_SIZE)
        );
        ((Button) mLayout.findViewById(R.id.pinfo_ffont_rom_id)).setOnClickListener(
                new NumCommandListener(PrinterInformation.GET_FFONT_ROM_ID)
        );

    }

    @Override
    public String onRequestSend() throws PrinterException {
        StringBuilder builder = new StringBuilder();
        switch (mType) {
            case INFO_BYTES:
                byte[] ret = mPrinterManager.getPrinterInformation(mPrinterInformation);
                for (byte b : ret) {
                    builder.append(String.format("%02X", b));
                }
                break;
            case INFO_STRING:
                String rets = mPrinterManager.getPrinterInformationString(mPrinterInformation);
                builder.append(rets);
                break;
            case INFO_NUMBER:
                int retn = mPrinterManager.getPrinterInformationNumber(mPrinterInformation);
                builder.append(retn);
                break;
        }
        return builder.toString();
    }

    private class CommandListener implements View.OnClickListener {
        PrinterInformation mPrinterInfo;

        public CommandListener(PrinterInformation prnInfo) {
            this.mPrinterInfo = prnInfo;
        }

        @Override
        public void onClick(View v) {
            setInfo(mPrinterInfo, INFO_BYTES);
            doRequest(true);
        }
    }

    private class StringCommandListener implements View.OnClickListener {
        PrinterInformation mPrinterInfo;

        public StringCommandListener(PrinterInformation prninfo) {
            this.mPrinterInfo = prninfo;
        }

        @Override
        public void onClick(View v) {
            setInfo(mPrinterInfo, INFO_STRING);
            doRequest(true);
        }
    }

    private class NumCommandListener implements View.OnClickListener {
        PrinterInformation mPrinterInfo;

        public NumCommandListener(PrinterInformation prninfo) {
            this.mPrinterInfo = prninfo;
        }

        @Override
        public void onClick(View v) {
            setInfo(mPrinterInfo, INFO_NUMBER);
            doRequest(true);
        }
    }

    private void setInfo(PrinterInformation p, int t) {
        mType = t;
        mPrinterInformation = p;
    }
}
