package com.seikoinstruments.sdk.mobileprinter.sample;

import android.app.Dialog;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.Spinner;

import com.seikoinstruments.sdk.mobileprinter.CharacterSet;
import com.seikoinstruments.sdk.mobileprinter.InternationalCharacterSet;
import com.seikoinstruments.sdk.mobileprinter.PrinterException;

public class SettingDialogFragment extends CommandDialogFragment {

    private EditText mEditTextSocketKeepingTime;
    private EditText mEditTextWriteTimeout;
    private EditText mEditTextResponseTimeout;

    private int mWriteTimeout = DEFAULT_WRITE_TIMEOUT;
    private int mResponseTimeout = DEFAULT_RESPONSE_TIMEOUT;
    private CharacterSet mCodePage;
    private InternationalCharacterSet mInternationalChar;

    private static final CharacterSet[] sTypeCodePage = new CharacterSet[] {
            CharacterSet.CODEPAGE_437,
            CharacterSet.CODEPAGE_KATAKANA,
            CharacterSet.CODEPAGE_850,
            CharacterSet.CODEPAGE_860,
            CharacterSet.CODEPAGE_863,
            CharacterSet.CODEPAGE_865,
            CharacterSet.CODEPAGE_857,
            CharacterSet.CODEPAGE_737,
            CharacterSet.CODEPAGE_1252,
            CharacterSet.CODEPAGE_866,
            CharacterSet.CODEPAGE_852,
            CharacterSet.CODEPAGE_858,
            CharacterSet.CODEPAGE_855,
            CharacterSet.CODEPAGE_864,
            CharacterSet.CODEPAGE_1250,
            CharacterSet.CODEPAGE_1251,
            CharacterSet.CODEPAGE_1253,
            CharacterSet.CODEPAGE_1254
    };

    private static final InternationalCharacterSet[] sTypeInternationalChar =
            new InternationalCharacterSet[] {
            InternationalCharacterSet.INT_CHAR_SET_USA,
            InternationalCharacterSet.INT_CHAR_SET_FRANCE,
            InternationalCharacterSet.INT_CHAR_SET_GERMANY,
            InternationalCharacterSet.INT_CHAR_SET_UNITED_KINGDOM,
            InternationalCharacterSet.INT_CHAR_SET_DENMARK_1,
            InternationalCharacterSet.INT_CHAR_SET_SWEDEN,
            InternationalCharacterSet.INT_CHAR_SET_ITALY,
            InternationalCharacterSet.INT_CHAR_SET_SPAIN_1,
            InternationalCharacterSet.INT_CHAR_SET_JAPAN,
            InternationalCharacterSet.INT_CHAR_SET_NORWAY,
            InternationalCharacterSet.INT_CHAR_SET_DENMARK_2,
            InternationalCharacterSet.INT_CHAR_SET_SPAIN_2,
            InternationalCharacterSet.INT_CHAR_SET_LATIN_AMERICA,
            InternationalCharacterSet.INT_CHAR_SET_ARABIA
    };

    private AdapterView.OnItemSelectedListener mCodePageListener =
            new AdapterView.OnItemSelectedListener() {
                @Override
                public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                    mCodePage = sTypeCodePage[position];
                }

                @Override
                public void onNothingSelected(AdapterView<?> parent) {
                    mCodePage = sTypeCodePage[0];
                }
            };

    private AdapterView.OnItemSelectedListener mInternationalCharListener =
            new AdapterView.OnItemSelectedListener() {
                @Override
                public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                    mInternationalChar = sTypeInternationalChar[position];
                }

                @Override
                public void onNothingSelected(AdapterView<?> parent) {
                    mInternationalChar = sTypeInternationalChar[0];
                }
            };

    private View.OnClickListener mApplyListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            try {
                mSocketKeepingTime = getSocketKeepingTime();
                mWriteTimeout = getWriteTimeout();
                mResponseTimeout = getResponseTimeout();
                mBtSecure = getBtSecure();
                applySetting();
            } catch (NumberFormatException e) {
                popDialog(e.getClass().getSimpleName(), e.getMessage());
            }
        }
    };

    public static SettingDialogFragment newInstance(ConnectionInformation info) {
        SettingDialogFragment fragment = new SettingDialogFragment();
        Bundle args = new Bundle();
        args.putSerializable(MainActivity.CONN_INFO, info);
        fragment.setArguments(args);

        return fragment;
    }

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        setLayout(getActivity().getLayoutInflater().inflate(R.layout.setting_dialog, null));
        return super.onCreateDialog(savedInstanceState);
    }

    @Override
    protected void setupWidgets() {
        //get property
        mSocketKeepingTime = getPropertyInt(KEY_SOCKET_KEEPING_TIME);
        mWriteTimeout = getPropertyInt(KEY_WRITE_TIMEOUT);
        mResponseTimeout = getPropertyInt(KEY_RESPONSE_TIMEOUT);
        mBtSecure = getPropertyBool(KEY_BT_SECURE);
        mCodePage = getPropertyCharacterSet(KEY_CHAR_SET);
        mInternationalChar = getPropertyIntCharacterSet(KEY_INT_CHAR_SET);

        // Socket Keeping Time
        mEditTextSocketKeepingTime =
                (EditText) mLayout.findViewById(R.id.et_setting_socket_keeping_time);
        mEditTextSocketKeepingTime.setText("" + mSocketKeepingTime);

        // Write timeout
        mEditTextWriteTimeout =
                (EditText) mLayout.findViewById(R.id.et_setting_write_timeout);
        mEditTextWriteTimeout.setText("" + mWriteTimeout);

        // Response timeout
        mEditTextResponseTimeout =
                (EditText) mLayout.findViewById(R.id.et_setting_response_timeout);
        mEditTextResponseTimeout.setText("" + mResponseTimeout);

        // Code page
        Spinner spCodePage = (Spinner) mLayout.findViewById(R.id.sp_setting_code_page);
        ArrayAdapter<CharSequence> codePageAdapter = ArrayAdapter.createFromResource(
                getActivity(), R.array.code_page, android.R.layout.simple_spinner_item);
        codePageAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spCodePage.setAdapter(codePageAdapter);
        spCodePage.setOnItemSelectedListener(mCodePageListener);
        spCodePage.setSelection(getPosition(mCodePage));

        // International character
        Spinner spInternationalChar =
                (Spinner) mLayout.findViewById(R.id.sp_setting_international_char);
        ArrayAdapter<CharSequence> internationalCharAdapter = ArrayAdapter.createFromResource(
                getActivity(), R.array.international_char,
                android.R.layout.simple_spinner_item);
        internationalCharAdapter.
                setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spInternationalChar.setAdapter(internationalCharAdapter);
        spInternationalChar.setOnItemSelectedListener(mInternationalCharListener);
        spInternationalChar.setSelection(getPosition(mInternationalChar));

        // Bluetooth secure
        if(mBtSecure) {
            ((RadioButton) mLayout.findViewById(R.id.rb_setting_bt_secure)).setChecked(true);
        } else {
            ((RadioButton) mLayout.findViewById(R.id.rb_setting_bt_insecure)).setChecked(true);
        }

        // print and cancel button
        ((Button) mLayout.findViewById(R.id.btn_setting_apply)).
                setOnClickListener(mApplyListener);
        ((Button) mLayout.findViewById(R.id.btn_setting_cancel)).
                setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismiss();
            }
        });
    }

    private void applySetting() {
        try {
            applySetting(mWriteTimeout, mResponseTimeout,
                    mCodePage, mInternationalChar);
            setProperty(mSocketKeepingTime, mWriteTimeout, mResponseTimeout, mBtSecure,
                    mCodePage, mInternationalChar);
            popInfoDialog(getString(R.string.setting_ok_msg));
        } catch (PrinterException e) {
            popPrinterExceptionDialog(e.getErrorCode());
        }
    }

    private int getPosition(CharacterSet c) {
        int pos = 0;

        for (int i = 0; i < sTypeCodePage.length; i++) {
            if (sTypeCodePage[i].equals(c)) {
                pos = i;
            }
        }
        return pos;
    }

    private int getPosition(InternationalCharacterSet c) {
        int pos = 0;

        for (int i = 0; i < sTypeInternationalChar.length; i++) {
            if (sTypeInternationalChar[i].equals(c)) {
                pos = i;
            }
        }
        return pos;
    }

    private int getSocketKeepingTime() {
        return Integer.parseInt(mEditTextSocketKeepingTime.getText().toString());
    }

    private int getWriteTimeout() {
        return Integer.parseInt(mEditTextWriteTimeout.getText().toString());
    }

    private int getResponseTimeout() {
        return Integer.parseInt(mEditTextResponseTimeout.getText().toString());
    }

    private boolean getBtSecure() {
        RadioButton btSecure =
                (RadioButton) mLayout.findViewById(R.id.rb_setting_bt_secure);
        RadioButton btInsecure =
                (RadioButton) mLayout.findViewById(R.id.rb_setting_bt_insecure);
        boolean ret;

        if (btSecure.isChecked()) {
            ret = true;
        } else if (btInsecure.isChecked()) {
            ret = false;
        } else {
            ret = true;
        }
        return ret;
    }
}
