package com.seikoinstruments.sdk.mobileprinter.sample;

import android.app.Dialog;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.Spinner;

import com.seikoinstruments.sdk.mobileprinter.Alignment;
import com.seikoinstruments.sdk.mobileprinter.Bold;
import com.seikoinstruments.sdk.mobileprinter.CharacterScale;
import com.seikoinstruments.sdk.mobileprinter.CharacterType;
import com.seikoinstruments.sdk.mobileprinter.PrinterException;
import com.seikoinstruments.sdk.mobileprinter.Reverse;
import com.seikoinstruments.sdk.mobileprinter.Rotate;
import com.seikoinstruments.sdk.mobileprinter.Underline;

public class TextFormattingDialogFragment extends CommandDialogFragment
                implements CommandDialogFragment.RequestSender {

    private String mText;
    private CharacterType mCharacterType = CharacterType.CHAR_TYPE_FONT_A;
    private CharacterScale mCharacterScale;
    private Underline mUnderline = Underline.CHAR_UNDERLINE_NONE;
    private Bold mBold;
    private Reverse mReverse;
    private Rotate mRotate;

    private static final CharacterScale[] sCharacterScale = new CharacterScale[]{
            CharacterScale.CHAR_SCALE_X1,
            CharacterScale.CHAR_SCALE_X2,
            CharacterScale.CHAR_SCALE_X3,
            CharacterScale.CHAR_SCALE_X4,
            CharacterScale.CHAR_SCALE_X5,
            CharacterScale.CHAR_SCALE_X6,
            CharacterScale.CHAR_SCALE_X7,
            CharacterScale.CHAR_SCALE_X8
    };

    private static final Rotate[] sRotate = new Rotate[]{
            Rotate.ROTATE_NONE,
            Rotate.ROTATE_90_TO_RIGHT,
            Rotate.ROTATE_90_TO_LEFT,
            Rotate.ROTATE_180,
    };

    private AdapterView.OnItemSelectedListener mCharacterScaleListener =
            new AdapterView.OnItemSelectedListener() {
        @Override
        public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
            mCharacterScale = sCharacterScale[position];
        }

        @Override
        public void onNothingSelected(AdapterView<?> parent) {
            mCharacterScale = CharacterScale.CHAR_SCALE_X1;
        }
    };

    private AdapterView.OnItemSelectedListener mRotateListener =
            new AdapterView.OnItemSelectedListener() {
        @Override
        public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
            mRotate = sRotate[position];
        }

        @Override
        public void onNothingSelected(AdapterView<?> parent) {
            mRotate = Rotate.ROTATE_NONE;
        }
    };

    private View.OnClickListener mPrintListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            mText = getText();
            mCharacterType = getCharacterType();
            mUnderline = getUnderline();
            mBold = getBold();
            mReverse = getReverse();
            doRequest(false);
        }
    };

    public static TextFormattingDialogFragment newInstance(ConnectionInformation info) {
        TextFormattingDialogFragment fragment = new TextFormattingDialogFragment();
        Bundle args = new Bundle();
        args.putSerializable(MainActivity.CONN_INFO, info);
        fragment.setArguments(args);

        return fragment;
    }

    @Override
    public Dialog onCreateDialog(Bundle savedIntanceState) {
        mRequestSender = this;
        setLayout(getActivity().getLayoutInflater().
                inflate(R.layout.text_formatting_dialog, null));
        return super.onCreateDialog(savedIntanceState);
    }

    @Override
    protected void setupWidgets() {
        // Scale
        Spinner spScale = (Spinner) mLayout.findViewById(R.id.sp_text_formatting_scale);
        ArrayAdapter<CharSequence> scaleAdapter = ArrayAdapter.createFromResource(
                getActivity(), R.array.scale_setting, android.R.layout.simple_spinner_item);
        scaleAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spScale.setAdapter(scaleAdapter);
        spScale.setOnItemSelectedListener(mCharacterScaleListener);

        // Rotate
        Spinner spRotation = (Spinner) mLayout.findViewById(R.id.sp_text_formatting_rotate);
        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(
                getActivity(), R.array.rotate_setting, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spRotation.setAdapter(adapter);
        spRotation.setOnItemSelectedListener(mRotateListener);

        // print and close button
        ((Button) mLayout.findViewById(R.id.btn_text_formatting_print)).
                setOnClickListener(mPrintListener);
        ((Button) mLayout.findViewById(R.id.btn_text_formatting_close)).
                setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismiss();
            }
        });
    }

    @Override
    public String onRequestSend() throws PrinterException {
        mPrinterManager.setStandardModeAlignment(Alignment.ALIGN_LEFT);
        mPrinterManager.setCharacterFormatting(mCharacterType,
                mCharacterScale, mCharacterScale, mUnderline, mBold, mReverse, mRotate);
        mPrinterManager.printText(mText);
        mPrinterManager.feedLine(1);
        return getString(R.string.print_ok_msg);
    }

    private String getText() {
        EditText text = (EditText) mLayout.findViewById(R.id.et_text_formatting_text);
        return text.getText().toString();
    }

    private CharacterType getCharacterType() {
        RadioButton fontA = (RadioButton) mLayout.findViewById(R.id.rb_text_formatting_fonta);
        RadioButton fontB = (RadioButton) mLayout.findViewById(R.id.rb_text_formatting_fontb);
        CharacterType ret;
        if (fontA.isChecked()) {
            ret =  CharacterType.CHAR_TYPE_FONT_A;
        } else if (fontB.isChecked()) {
            ret =  CharacterType.CHAR_TYPE_FONT_B;
        } else {
            ret =  CharacterType.CHAR_TYPE_FONT_CURRENT;
        }
        return ret;
    }

    private Underline getUnderline() {
        RadioButton none =
                (RadioButton) mLayout.findViewById(R.id.rb_text_formatting_underline_none);
        RadioButton dot1 =
                (RadioButton) mLayout.findViewById(R.id.rb_text_formatting_underline_1_dot);
        RadioButton dot2 =
                (RadioButton) mLayout.findViewById(R.id.rb_text_formatting_underline_2_dot);
        Underline ret;

        if (none.isChecked()) {
            ret = Underline.CHAR_UNDERLINE_NONE;
        } else if (dot1.isChecked()) {
            ret = Underline.CHAR_UNDERLINE_1DOT;
        } else if (dot2.isChecked()) {
            ret = Underline.CHAR_UNDERLINE_2DOT;
        } else {
            ret = Underline.CHAR_UNDERLINE_CURRENT;
        }
        return ret;
    }

    private Bold getBold() {
        CheckBox bold = (CheckBox) mLayout.findViewById(R.id.cb_text_formatting_bold);
        return bold.isChecked() ? Bold.CHAR_BOLD_ON : Bold.CHAR_BOLD_OFF;
    }

    private Reverse getReverse() {
        CheckBox reverse = (CheckBox) mLayout.findViewById(R.id.cb_text_formatting_reverse);
        return reverse.isChecked() ? Reverse.CHAR_REVERSE_ON : Reverse.CHAR_REVERSE_OFF;
    }

}
