package com.seikoinstruments.sdk.mobileprinter.sample;

import android.app.Dialog;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;

import com.seikoinstruments.sdk.mobileprinter.Alignment;
import com.seikoinstruments.sdk.mobileprinter.ErrorCorrect2Dcode;
import com.seikoinstruments.sdk.mobileprinter.Mode2Dcode;
import com.seikoinstruments.sdk.mobileprinter.ModuleSize2Dcode;
import com.seikoinstruments.sdk.mobileprinter.PrinterException;
import com.seikoinstruments.sdk.mobileprinter.Type2DCode;

public class TwoDimensionalBarcodeDialogFragment extends CommandDialogFragment
                        implements CommandDialogFragment.RequestSender {

    private EditText mEditTextData;
    private Spinner mSpinnerMode;
    private Spinner mSpinnerModuleSize;
    private EditText mEditTextModuleHeight;
    private EditText mEditTextColumn;
    private EditText mEditTextRow;
    private Spinner mSpinnerErrorCorrect;

    private Type2DCode mType;
    private String mData;
    private Mode2Dcode mMode;
    private ModuleSize2Dcode mModuleSize;
    private int mModuleHeight;
    private int mColumn;
    private int mRow;
    private ErrorCorrect2Dcode mErrorCorrect;

    private static final Type2DCode[] sType = new Type2DCode[] {
            Type2DCode.TYPE_2DCODE_QR_CODE,
            Type2DCode.TYPE_2DCODE_PDF417,
            Type2DCode.TYPE_2DCODE_DATA_MATRIX,
            Type2DCode.TYPE_2DCODE_MAXI_CODE,
            Type2DCode.TYPE_2DCODE_GS1_STACKED,
            Type2DCode.TYPE_2DCODE_GS1_OMNI_DIRECTIONAL,
            Type2DCode.TYPE_2DCODE_GS1_EXPANDED_STACKED
    };

    private static final Mode2Dcode[] sQrCodeMode = new Mode2Dcode[] {
            Mode2Dcode.MODE_2DCODE_QR_CODE_MODEL1,
            Mode2Dcode.MODE_2DCODE_QR_CODE_MODEL2
    };

    private static final Mode2Dcode[] sDataMatrixMode = new Mode2Dcode[] {
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_10_10,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_12_12,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_14_14,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_16_16,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_18_18,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_20_20,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_22_22,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_24_24,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_26_26,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_32_32,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_36_36,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_40_40,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_44_44,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_48_48,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_52_52,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_64_64,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_72_72,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_80_80,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_88_88,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_96_96,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_104_104,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_120_120,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_132_132,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_144_144,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_8_18,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_8_32,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_12_26,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_12_36,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_16_36,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_16_48,
            Mode2Dcode.MODE_2DCODE_DATA_MATRIX_AUTO
    };

    private static final Mode2Dcode[] sMaxiCodeMode = new Mode2Dcode[] {
            Mode2Dcode.MODE_2DCODE_MAXI_CODE_2,
            Mode2Dcode.MODE_2DCODE_MAXI_CODE_3,
            Mode2Dcode.MODE_2DCODE_MAXI_CODE_4,
            Mode2Dcode.MODE_2DCODE_MAXI_CODE_5,
    };

    private static final Mode2Dcode[] sPdf417Mode = new Mode2Dcode[] {
            Mode2Dcode.MODE_2DCODE_PDF417_STANDARD,
            Mode2Dcode.MODE_2DCODE_PDF417_COMPACT
    };

    private static final Mode2Dcode[] sGs1Mode = new Mode2Dcode[] {
            Mode2Dcode.MODE_2DCODE_NONE
    };

    private static final ModuleSize2Dcode[] sModuleSize = new ModuleSize2Dcode[] {
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_DEFAULT,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_2,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_3,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_4,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_5,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_6,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_7,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_8,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_9,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_10,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_11,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_12,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_13,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_14,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_15,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_16
    };

    private static final ModuleSize2Dcode[] sPdf417ModuleSize = new ModuleSize2Dcode[] {
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_DEFAULT,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_2,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_3,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_4,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_5,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_6,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_7,
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_8
    };

    private static final ModuleSize2Dcode[] sMaxiCodeModuleSize = new ModuleSize2Dcode[] {
            ModuleSize2Dcode.MODULE_SIZE_2DCODE_DEFAULT,
    };

    private static final ErrorCorrect2Dcode[] sQrCodeErrCorrect = new ErrorCorrect2Dcode[] {
            ErrorCorrect2Dcode.ERR_CORRECTION_2DCODE_QR_CODE_L,
            ErrorCorrect2Dcode.ERR_CORRECTION_2DCODE_QR_CODE_M,
            ErrorCorrect2Dcode.ERR_CORRECTION_2DCODE_QR_CODE_Q,
            ErrorCorrect2Dcode.ERR_CORRECTION_2DCODE_QR_CODE_H
    };

    private static final ErrorCorrect2Dcode[] sPdf417ErrCorrect = new ErrorCorrect2Dcode[] {
            ErrorCorrect2Dcode.ERR_CORRECTION_2DCODE_PDF417_0,
            ErrorCorrect2Dcode.ERR_CORRECTION_2DCODE_PDF417_1,
            ErrorCorrect2Dcode.ERR_CORRECTION_2DCODE_PDF417_2,
            ErrorCorrect2Dcode.ERR_CORRECTION_2DCODE_PDF417_3,
            ErrorCorrect2Dcode.ERR_CORRECTION_2DCODE_PDF417_4,
            ErrorCorrect2Dcode.ERR_CORRECTION_2DCODE_PDF417_5,
            ErrorCorrect2Dcode.ERR_CORRECTION_2DCODE_PDF417_6,
            ErrorCorrect2Dcode.ERR_CORRECTION_2DCODE_PDF417_7,
            ErrorCorrect2Dcode.ERR_CORRECTION_2DCODE_PDF417_8
    };

    private static final ErrorCorrect2Dcode[] sNoErrCorrect = new ErrorCorrect2Dcode[] {
            ErrorCorrect2Dcode.ERR_CORRECTION_2DCODE_NONE
    };

    private AdapterView.OnItemSelectedListener m2DcodeTypeListener =
            new AdapterView.OnItemSelectedListener() {
        @Override
        public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
            mType = sType[position];
            setupOptions(mType);
            replaceData(mType);
        }

        @Override
        public void onNothingSelected(AdapterView<?> parent) {
            mType = sType[0];
            setupOptions(mType);
        }

        private void setupOptions(Type2DCode type) {
            // Mode
            switch (type) {
                case TYPE_2DCODE_QR_CODE:
                    setAdapter(R.array.qrcode_mode, mSpinnerMode,
                            new ModeSelectedListener(sQrCodeMode));
                    break;
                case TYPE_2DCODE_DATA_MATRIX:
                    setAdapter(R.array.data_matrix_mode, mSpinnerMode,
                            new ModeSelectedListener(sDataMatrixMode));
                    break;
                case TYPE_2DCODE_MAXI_CODE:
                    setAdapter(R.array.maxi_code_mode, mSpinnerMode,
                            new ModeSelectedListener(sMaxiCodeMode));
                    break;
                case TYPE_2DCODE_PDF417:
                    setAdapter(R.array.pdf_417_mode, mSpinnerMode,
                            new ModeSelectedListener(sPdf417Mode));
                    break;
                case TYPE_2DCODE_GS1_STACKED:
                case TYPE_2DCODE_GS1_OMNI_DIRECTIONAL:
                case TYPE_2DCODE_GS1_EXPANDED_STACKED:
                    setAdapter(R.array.gs1_mode, mSpinnerMode, new ModeSelectedListener(sGs1Mode));
                    break;
            }

            // Module Size
            switch (type) {
                case TYPE_2DCODE_QR_CODE:
                case TYPE_2DCODE_DATA_MATRIX:
                case TYPE_2DCODE_GS1_STACKED:
                case TYPE_2DCODE_GS1_OMNI_DIRECTIONAL:
                case TYPE_2DCODE_GS1_EXPANDED_STACKED:
                    setAdapter(R.array.module_size, mSpinnerModuleSize,
                            new ModuleSizeSelectedListener(sModuleSize));
                    break;
                case TYPE_2DCODE_PDF417:
                    setAdapter(R.array.pdf417_module_size, mSpinnerModuleSize,
                            new ModuleSizeSelectedListener(sPdf417ModuleSize));
                    break;
                case TYPE_2DCODE_MAXI_CODE:
                    setAdapter(R.array.default_module_size, mSpinnerModuleSize,
                            new ModuleSizeSelectedListener(sMaxiCodeModuleSize));
                    break;
            }

            // Error Correct
            switch (type) {
                case TYPE_2DCODE_QR_CODE:
                    setAdapter(R.array.qrcode_err_correct, mSpinnerErrorCorrect,
                            new ErrorCorrectSelectedListener(sQrCodeErrCorrect));
                    break;
                case TYPE_2DCODE_PDF417:
                    setAdapter(R.array.pdf417_err_correct, mSpinnerErrorCorrect,
                            new ErrorCorrectSelectedListener(sPdf417ErrCorrect));
                    break;
                default:
                    setAdapter(R.array.no_err_correct, mSpinnerErrorCorrect,
                            new ErrorCorrectSelectedListener(sNoErrCorrect));
                    break;
            }
        }

        private void replaceData(Type2DCode code) {
            mEditTextModuleHeight.setText("0");
            mEditTextRow.setText("0");
            mEditTextColumn.setText("0");

            switch (code) {
                case TYPE_2DCODE_QR_CODE:
                    mEditTextData.setText("ABC0123456789");
                    break;
                case TYPE_2DCODE_DATA_MATRIX:
                    mEditTextData.setText("ABC0123456789");
                    mSpinnerMode.setSelection(30);    //MODE_2DCODE_DATA_MATRIX_AUTO
                    break;
                case TYPE_2DCODE_PDF417:
                    mEditTextData.setText("ABC0123456789");
                    mEditTextModuleHeight.
                            setText(String.valueOf(Type2DCode.PDF417_MODULE_HEIGHT_DEFAULT));
                    mEditTextColumn.setText(String.valueOf(Type2DCode.PDF417_COLUMN_AUTO));
                    mEditTextRow.setText(String.valueOf(Type2DCode.PDF417_ROW_AUTO));
                    break;
                case TYPE_2DCODE_MAXI_CODE:
                    mEditTextData.setText("111222333333333ABC0123456789DEF");
                    break;
                case TYPE_2DCODE_GS1_STACKED:
                    mEditTextData.setText("1230123456789");
                    break;
                case TYPE_2DCODE_GS1_OMNI_DIRECTIONAL:
                    mEditTextData.setText("1230123456789");
                    mEditTextModuleHeight.setText("33");
                    break;
                case TYPE_2DCODE_GS1_EXPANDED_STACKED:
                    mEditTextData.setText("1");
                    mEditTextColumn.setText("2");
                    break;
                default:
                    mEditTextData.setText("ABC0123456789");
                    break;
            }
        }

        private void setAdapter(int arrayId, Spinner spinner,
                                AdapterView.OnItemSelectedListener listener) {
            ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(
                    getActivity(), arrayId, android.R.layout.simple_spinner_item);
            adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
            spinner.setAdapter(adapter);
            spinner.setOnItemSelectedListener(listener);
        }
    };

    private View.OnClickListener mPrintListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            try {
                mData = mEditTextData.getText().toString();
                String moduleHeightStr = mEditTextModuleHeight.getText().toString();
                String columnStr = mEditTextColumn.getText().toString();
                String rowStr = mEditTextRow.getText().toString();
                mModuleHeight = Integer.parseInt(moduleHeightStr);
                mColumn = Integer.parseInt(columnStr);
                mRow = Integer.parseInt(rowStr);
                doRequest(false);
            } catch (NumberFormatException e) {
                popDialog(e.getClass().getSimpleName(), e.getMessage());
            }
        }
    };

    public static TwoDimensionalBarcodeDialogFragment newInstance(ConnectionInformation info) {
        TwoDimensionalBarcodeDialogFragment fragment = new TwoDimensionalBarcodeDialogFragment();
        Bundle args = new Bundle();
        args.putSerializable(MainActivity.CONN_INFO, info);
        fragment.setArguments(args);

        return fragment;
    }

    @Override
    public Dialog onCreateDialog(Bundle savedIntanceState) {
        mRequestSender = this;
        setLayout(getActivity().getLayoutInflater().
                inflate(R.layout.two_dimensional_barcode_dialog, null));
        return super.onCreateDialog(savedIntanceState);
    }

    @Override
    protected void setupWidgets() {
        // 2Dcode Type
        Spinner spType = (Spinner) mLayout.findViewById(R.id.sp_2dbarcode_type);
        ArrayAdapter<CharSequence> typeAdapter = ArrayAdapter.createFromResource(
                getActivity(), R.array.two_dimensional_barcode_type,
                android.R.layout.simple_spinner_item);
        typeAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spType.setAdapter(typeAdapter);
        spType.setOnItemSelectedListener(m2DcodeTypeListener);

        // Data
        mEditTextData = (EditText) mLayout.findViewById(R.id.et_2dbarcode_data);

        // Mode
        mSpinnerMode = (Spinner) mLayout.findViewById(R.id.sp_2dbarcode_mode);

        // Module Size
        mSpinnerModuleSize = (Spinner) mLayout.findViewById(R.id.sp_2dbarcode_module_size);

        // Module Height
        mEditTextModuleHeight =
                (EditText) mLayout.findViewById(R.id.et_2dbarcode_module_height);

        // Column
        mEditTextColumn = (EditText) mLayout.findViewById(R.id.et_2dbarcode_column);

        // Row
        mEditTextRow = (EditText) mLayout.findViewById(R.id.et_2dbarcode_row);

        // Error Correct
        mSpinnerErrorCorrect =
                (Spinner) mLayout.findViewById(R.id.sp_2dbarcode_error_correct);

        // print and close button
        ((Button) mLayout.findViewById(R.id.btn_2dbarcode_print)).
                setOnClickListener(mPrintListener);
        ((Button) mLayout.findViewById(R.id.btn_2dbarcode_close)).
                setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        dismiss();
                    }
                });
    }

    @Override
    public String onRequestSend() throws PrinterException {
        mPrinterManager.setStandardModeAlignment(Alignment.ALIGN_CENTER);
        mPrinterManager.print2DCode(mType, mData, mMode, mModuleSize,
                mModuleHeight, mColumn, mRow, mErrorCorrect);
        return getString(R.string.print_ok_msg);
    }

    private class ModeSelectedListener implements AdapterView.OnItemSelectedListener {
        private Mode2Dcode[] mModeList;

        public ModeSelectedListener(Mode2Dcode[] modes) {
            this.mModeList = modes;
        }

        @Override
        public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
            mMode = mModeList[position];
        }

        @Override
        public void onNothingSelected(AdapterView<?> parent) {
            mMode = mModeList[0];
        }
    }

    private class ModuleSizeSelectedListener implements AdapterView.OnItemSelectedListener {
        private ModuleSize2Dcode[] mModuleList;

        public ModuleSizeSelectedListener(ModuleSize2Dcode[] modules) {
            this.mModuleList = modules;
        }

        @Override
        public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
            mModuleSize = mModuleList[position];
        }

        @Override
        public void onNothingSelected(AdapterView<?> parent) {
            mModuleSize = mModuleList[0];
        }
    }

    private class ErrorCorrectSelectedListener implements AdapterView.OnItemSelectedListener {
        private ErrorCorrect2Dcode[] mErrorCorrectList;

        public ErrorCorrectSelectedListener(ErrorCorrect2Dcode[] ecs) {
            this.mErrorCorrectList = ecs;
        }

        @Override
        public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
            mErrorCorrect = mErrorCorrectList[position];
        }

        @Override
        public void onNothingSelected(AdapterView<?> parent) {
            mErrorCorrect = mErrorCorrectList[0];
        }
    }
}
